#ifndef FZSSH_VISIBILITY_CLIENT_HEADER
#define FZSSH_VISIBILITY_CLIENT_HEADER

#include "visibility.hpp"

#ifdef BUILDING_FZSSH_CLIENT
  #define FZSSH_CLIENT_PUBLIC_SYMBOL FZ_EXPORT_PUBLIC
  #define FZSSH_CLIENT_PRIVATE_SYMBOL FZ_EXPORT_PRIVATE
#else
  #define FZSSH_CLIENT_PUBLIC_SYMBOL FZ_IMPORT_SHARED
  #define FZSSH_CLIENT_PRIVATE_SYMBOL
#endif

#endif
